/*****************************************************************************
    neterror.h

    Diese Datei ist Teil des Programmpakets 'ACL-tftp'.
    Alle Rechte bei Andreas G. Schindler
                    Klappacher Str. 2i
                    64285 Darmstadt, Deutschland.

    Dieses Programm ist 'Freie Software'; Sie knnen es entsprechend den 
    Bedingungen der 'Allgemeinen ffentlichen GNU-Lizenz' (verffentlicht 
    durch die 'Free Software Foundation') weiter verbreiten und/oder 
    modifizieren. Massgeblich ist die Version 2 oder eine neuere Version
    dieser Lizenz. 

    Dieses Programm wird in der Hoffnung dass es ntzlich is weitergegeben,
    jedoch OHNE JEDE GARANTIE. Dieser Garantieauschluss bezieht sich auch
    auf Eigenschaften wie Marktfhigkeit oder Eignung zu einem bestimmten
    Zweck. Details finden sich in der 'Allgemeinen ffentlichen GNU-Lizenz'.

    Eine Kopie dieser Lizenz sollte in  Programmpaket, dem diese Datei
    entstammt, enthalten sein. Falls nicht, schreiben Sie an die 
    'Free Software Foundation, Inc.', 59 Temple Place, Suite 330, Boston, 
    MA  02111-1307  USA.


    This file is part of the 'ACL-tftp' Package
    Copyright (C) 1999-2003 by Andreas G. Schindler
                               Klappacher Str. 2i
                               Darmstadt, Germany.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*****************************************************************************/
#ifndef NETERROR_H
#define NETERROR_H

/*
	return codes for packet check routines
*/

#define PC_OK    0
#define PC_RETRY 1
#define PC_AGAIN 2
#define PC_ERROR 3
#define PC_UNKP  4

/*
	modifier codes for packet check routines
*/

#define BM_DATA         1
#define BM_ACK          2
#define BM_INIT		3
#define BM_OACK         4
#define BM_DOACK        5
#define BM_AOACK        6
#define BM_GRQ          7

/*
	error shortcuts
*/

#define sepWhere(WHERE) ((WHERE==NULL)? "" : _(" during "))
#define valWhere(WHERE) ((WHERE==NULL)? "" : WHERE)

#define logReceiveError(WHERE) \
    LOG_MSG1 (LOG_ERR, _("Receive error%s%s: %m"), \
              sepWhere(WHERE), valWhere(WHERE))

#define logSendError(WHERE) \
    LOG_MSG1 (LOG_ERR, _("Send error%s%s: %m"), \
              sepWhere(WHERE), valWhere(WHERE))

#define logTimeError(WHERE, TVP) \
    LOG_MSG2 (LOG_WARNING, _("Timeout (%.1fs)%s%s"),  \
              tv2double(TVP), sepWhere(WHERE), valWhere(WHERE))

#define logSelError(WHERE) \
    LOG_MSG1 (LOG_ERR, _("Select error%s%s: %m"), \
              sepWhere(WHERE), valWhere(WHERE))

#define logSelTimeout(WHERE, COUNT, TVP) \
    LOG_MSG2 (LOG_WARNING, _("Select timeout %d (%4.1fs)%s%s"), \
              COUNT, tv2double(TVP), sepWhere(WHERE), valWhere(WHERE))

#define logSelInter(WHERE) \
    LOG_MSG6 (LOG_DEBUG, _("Select interrupted%s%s"), \
              sepWhere(WHERE), valWhere(WHERE))

#define logShortPacket(VAL, WHERE) \
    LOG_MSG6 (LOG_DEBUG, _("Short packet received (%d bytes)%s%s"), \
              VAL, sepWhere(WHERE), valWhere(WHERE))

#define logGiantPacket(VAL, WHERE) \
    LOG_MSG6 (LOG_DEBUG, _("Giant packet received (%d bytes)%s%s"), \
              VAL, sepWhere(WHERE), valWhere(WHERE))

#define logUnknownPacket(PTYP, AWAITING) \
    LOG_MSG1 (LOG_ERR, _("Unknown packet type %d%s%s"), PTYP, \
              (AWAITING != NULL) ? _(" - awaiting ") : "",    \
              (AWAITING != NULL) ?       AWAITING    : "")

#define logUnexpectedPacket(PTYP, AWAITING) \
    LOG_MSG1 (LOG_ERR, _("Unexpected packet type %d (%s)%s%s"), \
              PTYP, opCode2txt(PTYP),                           \
              (AWAITING != NULL) ? _(" - awaiting ") : "",      \
              (AWAITING != NULL) ?       AWAITING    : "")

#define logPeerError(PTYP, ERRTXT) \
    LOG_MSG1 (LOG_ERR, _("Peer reports error %d '%s'"),  \
              (int)PTYP, (ERRTXT != NULL) ? ERRTXT : "")

#define logRetryError(WAITING4, RETRIES) \
    LOG_MSG1 (LOG_ERR, _("Too many retries (%d)%s%s"), RETRIES, \
              (WAITING4 != NULL) ? _(" - waiting for ") : "",   \
              (WAITING4 != NULL) ?       WAITING4    : "")

#define logTypeError(PACKETTYPE, WHERE) \
    LOG_MSG1 (LOG_ERR, _("Bad packet type %d (%s)%s%s"), \
              PACKETTYPE, opCode2txt(PACKETTYPE),       \
              sepWhere(WHERE), valWhere(WHERE))

#define logRepeatRequest(WHERE, SEQNO) \
    LOG_MSG4 (LOG_INFO, _("Block %hd repeat requested%s%s"), SEQNO, \
              sepWhere(WHERE), valWhere(WHERE))

#define logRetryCount(RETRY) \
    LOG_MSG6 (LOG_DEBUG, _("Retry %d"), RETRY)

/*
	packet check shortcuts
*/

#define checkGrq(REQUEST,PACKET, PACKETLEN) \
    checkRpack(REQUEST,PACKET,PACKETLEN,BM_GRQ)

#define checkData(REQUEST,PACKET, PACKETLEN) \
    checkRpack(REQUEST,PACKET,PACKETLEN,BM_DATA)

#define checkAck(REQUEST,PACKET,PACKETLEN) \
    checkRpack(REQUEST,PACKET,PACKETLEN,BM_ACK)

#define checkDataOack(REQUEST,PACKET,PACKETLEN) \
    checkRpack(REQUEST,PACKET,PACKETLEN,BM_DOACK)

#define checkAckOack(REQUEST,PACKET,PACKETLEN) \
    checkRpack(REQUEST,PACKET,PACKETLEN,BM_AOACK)

/*
	error logging & packet check functions
*/

void logSequenceError(char *what, int seq, int awaiting);
int retryPossible(tftpR_t *request, int errType, char *errText);
int checkRpack(tftpR_t *request, tftpD_t *packet, int packetLen, int okMask);
int checkSendError(tftpR_t *request, int rtc, int len, int okMask);
int checkSelError(tftpR_t *request, int rtc, int okMask);

#endif /* NETERROR_H */
